# © 2019 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

import logging
import time

from odoo import _, fields, models
from odoo.exceptions import UserError

_logger = logging.getLogger(__name__)


class AlfrescoDownloadZipWizard(models.TransientModel):
    _name = "alfresco.download.zip.wizard"
    _description = "Wizard de téléchargement de fichers"

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    # Odoo coops
    partner_select = fields.Selection(
        [
            ("query", "Par caractéristique"),
            ("partner", "Sélectionner des coopératives"),
        ],
        string="Type de sélection",
        default="query",
    )
    partner_ids = fields.Many2many(
        comodel_name="res.partner",
        string="Coopératives",
        domain=[("is_cooperative", "=", True)],
    )
    cooperative_form_ids = fields.Many2many(
        relation="alfresco_download_wizard_form_rel",
        comodel_name="res.partner.cooperative.form",
        column1="alfresco_wizard_form_id",
        column2="coop_form_id",
        string="Forme coopérative",
    )
    company_type_ids = fields.Many2many(
        relation="alfresco_download_wizard_type_rel",
        comodel_name="res.partner.company.type",
        column1="alfresco_wizard_type_id",
        column2="coop_type_id",
        string="Forme juridique",
    )
    is_cae = fields.Boolean(string="CAE")
    is_member = fields.Boolean(string="Adhérents")
    ur_id = fields.Many2many(
        relation="alfresco_download_wizard_ur_rel",
        comodel_name="union.regionale",
        column1="alfresco_wizard_ur_id",
        column2="ur_id",
        string="Union Régionale",
    )

    # Alfresco Metadata
    download_type = fields.Selection(
        [
            ("file", "Fichiers"),
            ("folder", "Dossiers"),
        ],
        string="Type de téléchargement",
        default="file",
    )
    year = fields.Char("Année")
    context = fields.Char("Contexte")
    type = fields.Char("Type")

    # ------------------------------------------------------
    # Button functions
    # ------------------------------------------------------
    def get_zip(self):
        download_type = (
            "crm:document" if self.download_type == "file" else "cmis:folder"
        )
        query = (
            "SELECT * FROM %s as d "
            "JOIN crm:organisme as o ON d.cmis:objectId = o.cmis:objectId WHERE "
        ) % (download_type,)
        # Sélection des coopératives
        coops = self._get_partner_ids()
        # Tableau des variables
        variables = [
            {
                "value": ", ".join("'" + str(p) + "'" for p in coops),
                "clause": "o.crm:id in (%s) ",
            },
        ]
        if self.download_type == "file":
            variables += [
                {"value": self.year, "clause": "d.crm:annee = '%s' "},
                {"value": self.context, "clause": "d.crm:contexte = '%s' "},
                {"value": self.type, "clause": "d.crm:type = '%s' "},
            ]
        # Suppression des variables nulles
        clauses = list(filter(None, [x if x.get("value") else None for x in variables]))
        if not clauses:
            raise UserError(_("La requête est trop large et non permise"))
        # Création de la requête
        for index, clause in enumerate(clauses):
            if index == 0:
                query += clause.get("clause") % clause.get("value")
            else:
                query += ("and " + clause.get("clause")) % clause.get("value")
        # Recherche des documents
        Alfresco = self.env["cgscop.alfresco"]
        search_result = Alfresco.alfresco_query_search(
            {
                "query": {
                    "query": query,
                    "language": "cmis",
                },
                "paging": {
                    "maxItems": "10000",
                },
            }
        )
        nodes = [x["entry"].get("id") for x in search_result["list"].get("entries")]
        if not nodes:
            raise UserError(_("Il n'y a pas de documents associés à cette recherche"))

        # Création du noeud de téléchargement
        download = Alfresco.alfresco_create_download(nodes)
        download_id = download["entry"].get("id")
        # Récupération des données du téléchargement
        download_get = Alfresco.alfresco_get_download(download_id)
        download_status = download_get["entry"].get("status")
        if download_get["entry"].get("totalBytes") > 838860800:
            # Suppression du noeud si la taille est trop élevée
            delete_download = Alfresco.alfresco_delete_download(download_id)
            logging.info("Alfresco delete download: %s" % delete_download)
            raise UserError(_("Le fichier à télécharger est trop volumineux (>800Mo)."))
        # Boucle de validation de fin de création du noeud
        while download_status != "DONE":
            time.sleep(0.5)
            download_get = Alfresco.alfresco_get_download(download_id)
            download_status = download_get["entry"].get("status")
            logging.info("Alfresco download status: %s" % download_get)

        return {
            "type": "ir.actions.act_url",
            "url": "/web/alfresco/download-zip?node=" + download_id,
            "target": "new",
        }

    # ------------------------------------------------------
    # Common functions
    # ------------------------------------------------------
    def _get_partner_ids(self):
        if self.partner_select == "partner":
            if not self.partner_ids:
                raise UserError(_("Vous devez sélectionner des coopératives."))
            return self.partner_ids.ids
        if self.partner_select == "query":
            domain = [("is_cooperative", "=", True)]
            if self.cooperative_form_ids:
                domain.append(
                    ("cooperative_form_id", "in", self.cooperative_form_ids.ids)
                )
            if self.company_type_ids:
                domain.append(
                    ("partner_company_type_id", "in", self.company_type_ids.ids)
                )
            if self.ur_id:
                domain.append(("ur_id", "in", self.ur_id.ids))
            if self.is_cae:
                domain.append(("cae", "=", True))
            if self.is_member:
                domain.append(("membership_status", "=", "member"))
            partner_ids = self.env["res.partner"].search(domain)
            return partner_ids.ids
